/** 
 * @file app.c
 */

#include "app.h"

APP_TIMER_DEF(m_afe_sample_timer);       /**< Periodic timer instance declaration */

/**
 * Local function to call power up sequence
 */
void power_up_sequence(void)
{
    nrf_delay_ms(10);                //generic board power up delay
}

/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}

/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{
    // Initialize timer module.
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);

    // Create timers.

    /* YOUR_JOB: Create any timers to be used by the application.
                 Below is an example of how to create a timer.
                 For every new timer needed, increase the value of the macro APP_TIMER_MAX_TIMERS by
                 one.
       ret_code_t err_code;
       err_code = app_timer_create(&m_app_timer_id, APP_TIMER_MODE_REPEATED, timer_timeout_handler);
       APP_ERROR_CHECK(err_code); */

    err_code = app_timer_create(&m_afe_sample_timer, APP_TIMER_MODE_REPEATED, Afe_Sampling_Task_Timer_Handler);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function to start basic app timers.
 */
static void timers_start(void)
{
    ret_code_t err_code;
    err_code = app_timer_start(m_afe_sample_timer, AFE_SAMPLING_PERIOD, NULL);
}

/**
 * Local function to call the inits for all of the peripheral drivers
 */
void peripheral_drivers_init(void)
{
    /*Initialize Nordic Peripheral Drivers***************************/

    //Not currently using ADC
    //Adc_Init();
    
    //Not currently using I2C
    //I2c_Init();

    Spi_Init();   //Communication with the AD7785 AFE

    Pwm_Init();   //Controls the two RGB LEDs

    /*Initialize TRI-SSM Peripheral Hardware Drivers*****************/
    bool config_error = false;

    config_error |= Ad7785_Init();    //Initialize AD7785 AFE

    if (config_error)
    {
        Led1_Color_Set(COLOR_RED);
        Led2_Color_Set(COLOR_RED);
    }else{
        Led1_Color_Set(COLOR_GREEN);
        Led2_Color_Set(COLOR_GREEN);
    }
}


/**
 * Global function to call to initialize the application structures/state machines/etc.
 */
void App_Init(void)
{
    power_up_sequence();

    log_init();

    power_management_init();

    timers_init();

    peripheral_drivers_init();

    Ble_Drivers_Init();

    APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);

    timers_start();
}